from babel import Locale

from molo.core.models import SiteLanguage

from unicore.content.models import Category, Page
from molo.core.content_import.helper import ImportError


class ContentImportValidation(object):
    errors = []

    def __init__(self, ws):
        self.ws = ws

    def get_main_language(self, locales):
        mains = [locale for locale in locales if locale.get('is_main')]

        if not mains:
            raise ImportError("No main languages have been given")
        elif len(mains) > 1:
            raise ImportError("Cannot have multiple main languages")
        else:
            return mains[0].get('locale')

    def is_validate_for(self, locales):
        self.errors = []
        main_language = self.get_main_language(locales)

        self.validate_wagtail_has_no_language(main_language)
        for l in locales:
            self.validate_translated_content_has_source(l)
            self.validate_page_primary_category_exists(l)
            self.validate_translated_content_source_exists(l, main_language)

        return self.errors

    def validate_wagtail_has_no_language(self, main_language):
        main_language = Locale.parse(main_language).language

        wagtail_main_language = SiteLanguage.objects.filter(
            is_main_language=True).first()
        if (wagtail_main_language and not
                wagtail_main_language.locale == main_language):
            self.errors.append({
                'type': 'wrong_main_language_exist_in_wagtail',
                'details': {
                    'lang': wagtail_main_language.get_locale_display(),
                    'selected_lang': Locale.parse(main_language).english_name
                }})

    def validate_translated_content_has_source(self, l):
        if not l.get('is_main'):
            child_language = l.get('locale')

            categories = self.ws.S(Category).filter(
                language=child_language).order_by('position')[:10000]
            for c in categories:
                if not c.source:
                    self.errors.append({
                        'type': 'no_source_found_for_category',
                        'details': {
                            'category': c.title,
                            'lang': Locale.parse(child_language).english_name
                        }})

            pages = self.ws.S(Page).filter(language=child_language)[:10000]
            for p in pages:
                if not p.source:
                    self.errors.append({
                        'type': 'no_source_found_for_page',
                        'details': {
                            'article': p.title,
                            'lang': Locale.parse(child_language).english_name
                        }})

    def validate_translated_content_source_exists(self, l, main_language):
        if not l.get('is_main'):
            child_language = l.get('locale')

            categories = self.ws.S(Category).filter(
                language=child_language).order_by('position')[:10000]
            for c in categories:
                if c.source and not self.validate_source_exists(
                        Category, c.source, main_language):
                    self.errors.append({
                        'type': 'category_source_not_exists',
                        'details': {
                            'category': c.title,
                            'lang': Locale.parse(child_language).english_name
                        }})

            pages = self.ws.S(Page).filter(language=child_language)[:10000]
            for p in pages:
                if p.source and not self.validate_source_exists(
                        Page, p.source, main_language):
                    self.errors.append({
                        'type': 'page_source_not_exists',
                        'details': {
                            'page': p.title,
                            'lang': Locale.parse(child_language).english_name
                        }})

    def validate_page_primary_category_exists(self, l):
        language = l.get('locale')
        pages = self.ws.S(Page).filter(language=language)[:10000]
        for p in pages:
            if p.primary_category and not self.validate_category_exists(
                    p.primary_category, language):
                self.errors.append({
                    'type': 'no_primary_category',
                    'details': {
                        'article': p.title,
                        'lang': Locale.parse(language).english_name
                    }})

    def validate_category_exists(self, uuid, locale):
        return self.ws.S(Category).filter(
            language=locale, uuid=uuid).count() > 0

    def validate_source_exists(self, cls, uuid, locale):
        return self.ws.S(cls).filter(
            language=locale, uuid=uuid).count() > 0



import pytest
import responses

from django.contrib.auth.models import User

from rest_framework.test import APIClient

from elasticgit.tests.base import ModelBaseTest

from molo.core.tests.base import MoloTestCaseMixin
from molo.core.content_import.tests.base import ElasticGitTestMixin


@pytest.mark.django_db
class ContentImportAPITestCase(
        ModelBaseTest, MoloTestCaseMixin, ElasticGitTestMixin):

    def setUp(self):
        self.client = APIClient()

    @responses.activate
    def test_get_repos(self):
        self.mock_get_repos_from_ucd()

        User.objects.create_superuser('testuser', 'testuser@email.com', '1234')
        self.client.login(username='testuser', password='1234')

        resp = self.client.get('/import/repos/')
        self.assertEquals(resp.data, {'repos': []})



import pytest

from elasticgit.tests.base import ModelBaseTest

from molo.core.models import SiteLanguage
from molo.core.tests.base import MoloTestCaseMixin
from molo.core.content_import.tests.base import ElasticGitTestMixin
from molo.core.content_import.validation import ContentImportValidation
from molo.core.content_import.helper import ImportError

from unicore.content import models as eg_models


@pytest.mark.django_db
class ContentImportValidationTestCase(
        ModelBaseTest, MoloTestCaseMixin, ElasticGitTestMixin):

    def setUp(self):
        self.mk_main()
        self.workspace = self.mk_workspace()

        self.workspace.setup_custom_mapping(eg_models.Localisation, {
            'properties': {
                'locale': {
                    'type': 'string',
                    'index': 'not_analyzed',
                }
            }
        })

        self.workspace.setup_custom_mapping(eg_models.Category, {
            'properties': {
                'uuid': {'type': 'string', 'index': 'not_analyzed'},
                'language': {'type': 'string', 'index': 'not_analyzed'},
                'position': {'type': 'integer', 'index': 'not_analyzed'},
            }
        })
        self.workspace.setup_custom_mapping(eg_models.Page, {
            'properties': {
                'uuid': {'type': 'string', 'index': 'not_analyzed'},
                'language': {'type': 'string', 'index': 'not_analyzed'},
                'position': {'type': 'integer', 'index': 'not_analyzed'},
            }
        })

    def test_that_an_ImportError_is_raised_for_invalid_main_languages(self):
        error_occured = False
        try:
            ContentImportValidation(self.workspace).is_validate_for([
                {'locale': 'eng_GB', 'site_language': 'en', 'is_main': False},
                {'locale': 'spa_ES', 'site_language': 'es', 'is_main': False}
            ])
        except ImportError:
            error_occured = True

        self.assertTrue(error_occured)

        error_occured = False
        try:
            ContentImportValidation(self.workspace).is_validate_for([
                {'locale': 'eng_GB', 'site_language': 'en', 'is_main': True},
                {'locale': 'spa_ES', 'site_language': 'es', 'is_main': True}
            ])
        except ImportError:
            error_occured = True

        self.assertTrue(error_occured)

    def test_language_validation(self):
        self.english = SiteLanguage.objects.create(
            locale='en',
        )

        error = ContentImportValidation(self.workspace).is_validate_for([
            {'locale': 'eng_GB', 'site_language': 'en', 'is_main': False},
            {'locale': 'spa_ES', 'site_language': 'es', 'is_main': True}])

        self.assertEquals(error[0]['type'],
                          'wrong_main_language_exist_in_wagtail')

    def test_import_validation(self):
        lang1 = eg_models.Localisation({'locale': 'eng_GB'})
        lang2 = eg_models.Localisation({'locale': 'spa_ES'})

        self.workspace.save(lang1, 'Added english language')
        self.workspace.save(lang2, 'Added french language')

        [eng_cat] = self.create_categories(
            self.workspace, locale='eng_GB', count=1)

        # spanish section with valid source
        self.create_categories(
            self.workspace, title='spanish cat', locale='spa_ES',
            source=eng_cat.uuid, count=1)

        [eng_page] = self.create_pages(
            self.workspace, locale='eng_GB', count=1)

        # spanish page with valid source
        self.create_pages(
            self.workspace, title='spanish page', locale='spa_ES',
            source=eng_page.uuid, count=1)

        # section with invalid source
        self.create_categories(
            self.workspace, locale='spa_ES',
            source='an-invalid-uuid', count=1)

        # page with invalid source
        self.create_pages(
            self.workspace, count=1, locale='spa_ES',
            source='an-invalid-uuid')

        # section with no source
        self.create_categories(
            self.workspace, locale='spa_ES', source=None, count=1)

        # traslated page without a source and an invalid primary category
        self.create_pages(
            self.workspace, source=None, count=1, locale='spa_ES',
            primary_category='an-invalid-primary-category')

        # main language page with invalid primary category
        self.create_pages(
            self.workspace, count=1, locale='eng_GB',
            primary_category='an-invalid-uuid')

        errors = ContentImportValidation(self.workspace).is_validate_for([
            {'locale': 'eng_GB', 'site_language': 'en', 'is_main': True},
            {'locale': 'spa_ES', 'site_language': 'es', 'is_main': False}])

        self.assertEquals(errors[0]['type'], 'no_primary_category')
        self.assertEquals(errors[1]['type'], 'no_source_found_for_category')
        self.assertEquals(errors[2]['type'], 'no_source_found_for_page')
        self.assertEquals(errors[3]['type'], 'no_primary_category')
        self.assertEquals(errors[4]['type'], 'category_source_not_exists')
        self.assertEquals(errors[5]['type'], 'page_source_not_exists')
        self.assertEquals(len(errors), 6)



import mock
import pytest

from elasticgit.tests.base import ModelBaseTest

from molo.core.models import SiteLanguage, SectionPage, ArticlePage, FooterPage
from molo.core.tests.base import MoloTestCaseMixin
from molo.core.content_import.tests.base import ElasticGitTestMixin
from molo.core.content_import.helper import ContentImportHelper, ImportError

from unicore.content import models as eg_models

from wagtail.wagtailimages.tests.utils import get_test_image_file


@pytest.mark.django_db
class ContentImportTestCase(
        ModelBaseTest, MoloTestCaseMixin, ElasticGitTestMixin):

    def setUp(self):
        self.english = SiteLanguage.objects.create(
            locale='en',
        )
        self.spanish = SiteLanguage.objects.create(
            locale='es',
        )
        self.mk_main()
        self.workspace = self.mk_workspace()

        self.workspace.setup_custom_mapping(eg_models.Localisation, {
            'properties': {
                'locale': {
                    'type': 'string',
                    'index': 'not_analyzed',
                }
            }
        })

        self.workspace.setup_custom_mapping(eg_models.Category, {
            'properties': {
                'language': {'type': 'string', 'index': 'not_analyzed'},
                'position': {'type': 'integer', 'index': 'not_analyzed'},
            }
        })
        self.workspace.setup_custom_mapping(eg_models.Page, {
            'properties': {
                'language': {'type': 'string', 'index': 'not_analyzed'},
                'position': {'type': 'integer', 'index': 'not_analyzed'},
            }
        })

    def test_import_sections_for_primary_language(self):
        lang1 = eg_models.Localisation({'locale': 'eng_GB'})
        lang2 = eg_models.Localisation({'locale': 'spa_ES'})

        self.workspace.save(lang1, 'Added english language')
        self.workspace.save(lang2, 'Added french language')

        [cat_eng_1, cat_eng_2] = self.create_categories(
            self.workspace, locale='eng_GB', count=2)
        [cat_spa_1, cat_spa_2] = self.create_categories(
            self.workspace, locale='spa_ES', count=2)

        # section with invalid source
        self.create_categories(
            self.workspace, locale='spa_ES', source='an-invalid-uuid', count=1)
        # section with no source
        self.create_categories(
            self.workspace, locale='spa_ES', source=None, count=1)

        self.workspace.save(cat_spa_1.update({
            'source': cat_eng_1.uuid,
            'position': 4,
        }), 'Added source to category.')
        self.workspace.save(cat_spa_2.update({
            'source': cat_eng_2.uuid,
            'position': 4,
        }), 'Added source to category.')

        en_pages1 = self.create_pages(
            self.workspace, count=10, locale='eng_GB',
            primary_category=cat_eng_1.uuid)

        en_pages2 = self.create_pages(
            self.workspace, count=10, locale='eng_GB',
            primary_category=cat_eng_2.uuid)

        # main language page with invalid primary category
        self.create_pages(
            self.workspace, count=1, locale='eng_GB',
            primary_category='an-invalid-uuid')

        en_footer_pages = self.create_pages(
            self.workspace, count=2, locale='eng_GB',
            primary_category=None)

        es_pages1 = self.create_pages(
            self.workspace, count=10, locale='spa_ES',
            primary_category=cat_eng_1.uuid)

        es_pages2 = self.create_pages(
            self.workspace, count=10, locale='spa_ES',
            primary_category=cat_eng_2.uuid)

        es_footer_pages = self.create_pages(
            self.workspace, count=2, locale='spa_ES',
            primary_category=None)

        # translation page without a source and primary category
        self.create_pages(
            self.workspace, count=1, locale='spa_ES',
            primary_category=None)

        # page with linked page
        [page_en] = self.create_pages(
            self.workspace, count=1, locale='eng_GB',
            primary_category=cat_eng_1.uuid)
        [page_with_linked_page] = self.create_pages(
            self.workspace, count=1, locale='eng_GB',
            linked_pages=[page_en.uuid], primary_category=cat_eng_1.uuid)

        for i in range(0, 10):
            self.workspace.save(es_pages1[i].update({
                'source': en_pages1[i].uuid,
                'author_tags': ['love'],
            }), 'Added author_tags and source to page.')

            self.workspace.save(es_pages2[i].update({
                'source': en_pages2[i].uuid,
            }), 'Added source to page.')

        for i in range(0, 2):
            self.workspace.save(es_footer_pages[i].update({
                'source': en_footer_pages[i].uuid,
            }), 'Added source to page.')

        self.assertEquals(
            self.workspace.S(eg_models.Category).all().count(), 6)

        self.assertEquals(
            self.workspace.S(eg_models.Page).all().count(), 48)
        self.assertEquals(
            self.workspace.S(eg_models.Localisation).all().count(), 2)

        self.assertEquals(SectionPage.objects.all().count(), 0)
        self.assertEquals(ArticlePage.objects.all().count(), 0)

        ContentImportHelper(self.workspace).import_content_for([
            {'locale': 'eng_GB', 'site_language': 'en', 'is_main': True},
            {'locale': 'spa_ES', 'site_language': 'es', 'is_main': False}])

        self.assertEquals(SectionPage.objects.all().count(), 4)
        self.assertEquals(ArticlePage.objects.all().count(), 46)
        self.assertEquals(
            ArticlePage.objects.filter(metadata_tags__name='love').count(), 10)

        page = ArticlePage.objects.get(uuid=page_with_linked_page.uuid)
        linked_page = ArticlePage.objects.get(uuid=page_en.uuid)
        self.assertEquals(page.body.stream_data[2],
                          {u'type': u'page', u'value': linked_page.pk})

        # run import twice
        ContentImportHelper(self.workspace).import_content_for([
            {'locale': 'eng_GB', 'site_language': 'en', 'is_main': True},
            {'locale': 'spa_ES', 'site_language': 'es', 'is_main': False}])

        self.assertEquals(SectionPage.objects.all().count(), 4)
        self.assertEquals(ArticlePage.objects.all().count(), 46)
        self.assertEquals(FooterPage.objects.all().count(), 4)

        # check that the main language no longer
        # needs to be first in the list of locales
        ContentImportHelper(self.workspace).import_content_for([
            {'locale': 'eng_GB', 'site_language': 'en', 'is_main': False},
            {'locale': 'spa_ES', 'site_language': 'es', 'is_main': True}])
        self.assertEquals(str(SiteLanguage.objects.filter(
            is_main_language=True).first()), 'English')
        self.assertEquals(str(SiteLanguage.objects.filter(
            is_main_language=False).first()), 'Spanish')

        # check that an ImportError is raised if
        # locales has more than one language with is_main == True
        error_occured = False
        try:
            ContentImportHelper(self.workspace).import_content_for([
                {'locale': 'eng_GB', 'site_language': 'en', 'is_main': True},
                {'locale': 'spa_ES', 'site_language': 'es', 'is_main': True}])
        except ImportError:
            error_occured = True
        self.assertTrue(error_occured)

        # check that an ImportError is raised
        # if none of the locales have is_main == True
        error_occured = False
        try:
            ContentImportHelper(self.workspace).import_content_for([
                {'locale': 'eng_GB', 'site_language': 'en', 'is_main': False},
                {'locale': 'spa_ES', 'site_language': 'es', 'is_main': False}])
        except ImportError:
            error_occured = True

        self.assertTrue(error_occured)

    def test_unknown_locale(self):
        lang1 = eg_models.Localisation({'locale': 'eng_GB'})
        lang2 = eg_models.Localisation({'locale': 'Zre_ZR'})

        self.workspace.save(lang1, 'Added english language')
        self.workspace.save(lang2, 'Added a language with unknown locale')

        [cat_eng_1, cat_eng_2] = self.create_categories(
            self.workspace, locale='eng_GB', count=2)
        [cat_Zre_1, cat_Zre_2] = self.create_categories(
            self.workspace, locale='Zre_ZR', count=2)

        locales, errors = ContentImportHelper(self.workspace).parse_locales()
        self.assertEquals(errors, [
            {'type': 'unknown_locale', 'details': {'locale': u'Zre_ZR'}}])

    @mock.patch('molo.core.content_import.get_image.get_thumbor_image_file')
    def test_image_import(self, mock_get_thumbor_image_file):
        mock_get_thumbor_image_file.return_value = get_test_image_file()

        lang1 = eg_models.Localisation({'locale': 'eng_GB'})
        self.workspace.save(lang1, 'Added english language')

        [cat_eng] = self.create_categories(
            self.workspace, locale='eng_GB', count=1)

        [en_page] = self.create_pages(
            self.workspace, count=1, locale='eng_GB',
            primary_category=cat_eng.uuid,
            image_host='http://thumbor', image='some-uuid-for-the-image')

        ContentImportHelper(self.workspace).import_content_for([
            {'locale': 'eng_GB', 'site_language': 'en', 'is_main': True}])

        self.assertEquals(ArticlePage.objects.all().count(), 1)
        self.assertEquals(
            ArticlePage.objects.all().first().image.title,
            'some-uuid-for-the-image')



import requests

from django.conf import settings
from elasticgit.workspace import RemoteWorkspace


from molo.core.content_import.helper import (
    ContentImportHelper)
from molo.core.content_import.validation import ContentImportValidation

from rest_framework.decorators import (
    api_view, authentication_classes, permission_classes)
from rest_framework.response import Response
from rest_framework.authentication import (
    SessionAuthentication, BasicAuthentication)
from rest_framework.permissions import IsAuthenticated

from unicore.content.models import Localisation, Category, Page


@api_view(['GET'])
def get_repos(request):
    response = requests.get(
        '%s/repos.json' % settings.UNICORE_DISTRIBUTE_API).json()
    return Response({
        'repos': [repo.get('name') for repo in response]})


@api_view(['GET'])
def get_repo_languages(request, name):
    ws = RemoteWorkspace('%s/repos/%s.json' % (
        settings.UNICORE_DISTRIBUTE_API, name))
    ws.sync(Localisation)
    locales, errors = ContentImportHelper(ws).parse_locales()

    return Response({
        'locales': locales,
        'errors': errors
    })


@api_view(['POST'])
@authentication_classes((SessionAuthentication, BasicAuthentication))
@permission_classes((IsAuthenticated,))
def import_content(request, name):
    ws = RemoteWorkspace('%s/repos/%s.json' % (
        settings.UNICORE_DISTRIBUTE_API, name))
    ws.sync(Localisation)
    ws.sync(Category)
    ws.sync(Page)

    # create wagtail content
    locales = request.data.get('locales')
    errors = ContentImportValidation(ws).is_validate_for(locales)

    if errors:
        return Response(status=422, data={'errors': errors})
    else:
        ContentImportHelper(ws).import_content_for(locales)
        return Response()


@api_view(['POST'])
@authentication_classes((SessionAuthentication, BasicAuthentication))
@permission_classes((IsAuthenticated,))
def import_validate(request, name):
    ws = RemoteWorkspace('%s/repos/%s.json' % (
        settings.UNICORE_DISTRIBUTE_API, name))
    ws.sync(Localisation)
    ws.sync(Category)
    ws.sync(Page)

    # validate import content
    locales = request.data.get('locales')
    errors = ContentImportValidation(ws).is_validate_for(locales)
    if errors:
        return Response(status=422, data={'errors': errors})
    else:
        return Response()



import json
import responses

from django.conf import settings

from wagtail.wagtailcore.utils import cautious_slugify
from unicore.content.models import Page, Category, Localisation


class ElasticGitTestMixin(object):

    def create_category_data_iter(self, count=2, locale='eng_UK', **kwargs):
        for i in range(count):
            data = {}
            data.update({
                'title': u'Test Category %s' % (i,),
                'language': locale
            })
            data.update(kwargs)
            data.update({
                'slug': cautious_slugify(data['title'])
            })
            yield data, i

    def create_categories(
            self, workspace, count=2, locale='eng_UK', **kwargs):
        categories = []
        for data, i in self.create_category_data_iter(
                count=count, locale=locale, **kwargs):
            category = Category(data)
            workspace.save(
                category, u'Added category %s.' % (i,))
            categories.append(category)

        workspace.refresh_index()
        return categories

    def create_page_data_iter(self, count=2, locale='eng_UK', **kwargs):
        for i in range(count):
            data = {}
            data.update({
                'title': u'Test Page %s' % (i,),
                'content': u'this is sample content for pg %s' % (i,),
                'language': locale
            })
            data.update(kwargs)
            data.update({
                'slug': cautious_slugify(data['title'])
            })
            yield data, i

    def create_pages(
            self, workspace, count=2, locale='eng_UK', **kwargs):
        pages = []
        for data, i in self.create_page_data_iter(
                count=count, locale=locale, **kwargs):
            page = Page(data)
            workspace.save(page, message=u'Added page %s.' % (i,))
            pages.append(page)

        workspace.refresh_index()
        return pages

    def create_localisation(self, workspace, locale='eng_UK', **kwargs):
        data = {'locale': locale}
        data.update(kwargs)
        loc = Localisation(data)
        workspace.save(loc, u'Added localisation %s.' % (locale,))
        workspace.refresh_index()
        return loc

    def mock_get_repos_from_ucd(self, data=[], status=200):
        responses.add(
            responses.GET, '%s/repos.json' % settings.UNICORE_DISTRIBUTE_API,
            body=json.dumps(data),
            content_type="application/json",
            status=status)



import json

from babel import Locale
from babel.core import UnknownLocaleError

from molo.core.models import (
    Main, SiteLanguage, PageTranslation, SectionPage, ArticlePage, FooterPage)
from molo.core.content_import.get_image import get_image_file

from unicore.content.models import Localisation, Category, Page


class ImportError(Exception):
    """Raised when there is an error importing"""


class ContentImportHelper(object):

    def __init__(self, ws):
        self.ws = ws

    def get_or_create(self, cls, obj, parent):
        if cls.objects.filter(uuid=obj.uuid).exists():
            return cls.objects.get(uuid=obj.uuid)

        instance = cls(uuid=obj.uuid, title=obj.title)
        parent.add_child(instance=instance)
        print 'created', obj.title
        return instance

    def get_or_create_translation(self, cls, obj, parent, language):
        if cls.objects.filter(uuid=obj.uuid).exists():
            return cls.objects.get(uuid=obj.uuid)

        instance = cls(uuid=obj.uuid, title=obj.title)
        parent.add_child(instance=instance)
        language_relation = instance.languages.first()
        language_relation.language = language
        language_relation.save()
        print 'created translation', obj.title
        return instance

    def import_section_content(self, c, site_language):
        main = Main.objects.all().first()
        if site_language.is_main_language:
            section = self.get_or_create(SectionPage, c, main)
        else:
            section = self.get_or_create_translation(
                SectionPage, c, main, site_language)

        section.description = c.subtitle
        if c.image_host and c.image:
            section.image = get_image_file(c.image_host, c.image)

        section.save_revision().publish()

        return section

    def import_page_content(self, p, site_language):
        if site_language.is_main_language:
            if p.primary_category:
                try:
                    section = SectionPage.objects.get(
                        uuid=p.primary_category)
                    page = self.get_or_create(ArticlePage, p, section)
                except SectionPage.DoesNotExist:
                    print "couldn't find primary category ", \
                        p.primary_category, \
                        SectionPage.objects.all().values('uuid')
                    return None
            else:
                # special case for articles with no primary category
                # this assumption is probably wrong..
                # but we have no where else to put them
                main = Main.objects.all().first()
                page = self.get_or_create(FooterPage, p, main)
        else:
            try:
                main_instance = ArticlePage.objects.get(
                    uuid=p.source).specific
                page = self.get_or_create_translation(
                    main_instance.__class__, p, main_instance.get_parent(),
                    site_language)
                PageTranslation.objects.get_or_create(
                    page=main_instance,
                    translated_page=page)
            except ArticlePage.DoesNotExist:
                print "No source found for: ", p.source, (
                    ArticlePage.objects.all().values('uuid'))
                return None

        page.subtitle = p.subtitle
        page.body = json.dumps([
            {'type': 'paragraph', 'value': p.description},
            {'type': 'paragraph', 'value': p.content}
        ])
        is_featured = p.featured if p.featured else False
        is_featured_in_category = p.featured_in_category \
            if p.featured_in_category else False

        page.featured_in_latest = is_featured
        page.featured_in_homepage = is_featured_in_category
        for tag in p.author_tags:
            page.metadata_tags.add(tag)
        if p.image_host and p.image:
            page.image = get_image_file(p.image_host, p.image)

        page.save_revision().publish()

        return page

    def update_pages_with_linked_page_field(self):
        for p in self.ws.S(Page).all()[:10000]:
            if p.linked_pages:
                for lp in p.linked_pages:
                    try:
                        page = ArticlePage.objects.get(uuid=p.uuid)
                        page.body.stream_data.append(
                            {u'type': u'page',
                             u'value': ArticlePage.objects.get(uuid=lp).pk})
                        page.save_revision().publish()
                    except ArticlePage.DoesNotExist:
                        print 'Linked page does not exist %s' % lp

    def import_categories_for_child_language(
            self, category, selected_locale, site_language):
        if category.source:
            try:
                parent = SectionPage.objects.get(
                    uuid=category.source)
                translated_section = self.import_section_content(
                    category, site_language)
                PageTranslation.objects.get_or_create(
                    page=parent,
                    translated_page=translated_section)
            except SectionPage.DoesNotExist:
                print "couldn't find", category.source, (
                    SectionPage.objects.all().values('uuid'))
        else:
            print "no source found for: ", category.source, (
                SectionPage.objects.all().values('uuid'))

    @classmethod
    def get_main_language(cls, locales):
        mains = [locale for locale in locales if locale.get('is_main')]

        if not mains:
            raise ImportError("No main languages have been given")
        elif len(mains) > 1:
            raise ImportError("Cannot have multiple main languages")
        else:
            return mains[0]

    def import_content_for(self, locales):
        main = self.get_main_language(locales)
        locales = [main] + [
            locale for locale in locales if not locale.get('is_main')]
        for selected_locale in locales:
            site_language, _ = SiteLanguage.objects.get_or_create(
                locale=Locale.parse(selected_locale.get('locale')).language,
                is_main_language=selected_locale.get('is_main'))

            self.import_all_categories(site_language, selected_locale)
            self.import_all_pages(site_language, selected_locale)

        self.update_pages_with_linked_page_field()

    def import_all_categories(self, site_language, selected_locale):
        category_qs = self.ws.S(Category).filter(
            language=selected_locale.get('locale')
        ).order_by('position')[:10000]
        # S() only returns 10 results if you don't ask for more
        if site_language.is_main_language:
            for c in category_qs:
                self.import_section_content(c, site_language)
        else:
            for c in category_qs:
                self.import_categories_for_child_language(
                    c, selected_locale, site_language)

    def import_all_pages(self, site_language, selected_locale):
        for p in self.ws.S(Page).filter(
            language=selected_locale.get('locale')
        ).order_by('position')[:10000]:
            # S() only returns 10 results if you don't ask for more

            self.import_page_content(p, site_language)

    def parse_locales(self):
        locales = []
        errors = []

        for l in self.ws.S(Localisation).all():
            try:
                locales.append({
                    'locale': l.locale,
                    'name': Locale.parse(l.locale).english_name
                })
            except UnknownLocaleError:
                errors.append({
                    'type': 'unknown_locale',
                    'details': {'locale': l.locale}
                })

        return locales, errors



import pytest

from django.test import TestCase
from django.core.urlresolvers import reverse

from molo.core.tests.base import MoloTestCaseMixin


@pytest.mark.django_db
class TestImportFromGit(TestCase, MoloTestCaseMixin):

    def setUp(self):
        self.mk_main()
        self.user = self.login()

    def test_wagtail_has_import_menu_item(self):
        response = self.client.get(reverse('import-from-git'))
        self.assertContains(
            response, 'Import content from a Universal Core site')
